/**
  \file ReservoirFortranFunc.cpp
 */


/*
<func2doc>

</func2doc>
 */


#include <openfluid/base.hpp>
#include <openfluid/core.hpp>

// specific Fortran
#include <openfluid/tools/FortranCPP.hpp>


// =====================================================================
// =====================================================================


DECLARE_PLUGIN_HOOKS;


// =====================================================================
// =====================================================================


BEGIN_SIGNATURE_HOOK

DECLARE_SIGNATURE_ID("training.encapsulation.reservoir-fortran");
DECLARE_SIGNATURE_NAME("water storage modelling - fortran language");
DECLARE_SIGNATURE_DESCRIPTION("this fonction distributes a water height in the different layers of the unsaturated zone");

DECLARE_SIGNATURE_VERSION("12.03");
DECLARE_SIGNATURE_SDKVERSION;
DECLARE_SIGNATURE_STATUS(openfluid::base::EXPERIMENTAL);

DECLARE_SIGNATURE_DOMAIN("unsaturated zone");
DECLARE_SIGNATURE_PROCESS("soil water flow");
DECLARE_SIGNATURE_METHOD("capacitive modelling");
DECLARE_SIGNATURE_AUTHORNAME("Crevoisier D, Dagès C.");
DECLARE_SIGNATURE_AUTHOREMAIL("david.crevoisier@supagro.inra.fr, cecile.dages@supagro.inra.fr");

// Input data
DECLARE_REQUIRED_INPUTDATA("th_cc","SU","Field capacity","m3/m3")
DECLARE_REQUIRED_INPUTDATA("thetai","SU","Initial water content","m3/m3")
DECLARE_REQUIRED_INPUTDATA("dz","SU","Soil layer depth","m")

// Variables
DECLARE_PRODUCED_VAR("water.surf-uz.theta[]","SU","Soil moisture profile","m3/m3")
DECLARE_PRODUCED_VAR("water.surf-uz.drainage","SU","Drainage below soil profile","m")
DECLARE_PRODUCED_VAR("water.surf-uz.storage","SU","Water storage in the whole soil column","m")
DECLARE_REQUIRED_VAR("water.atm-surf.H.rain","SU","Produced variable, distributed on SU - rainfall height on each SU by time step","m")


END_SIGNATURE_HOOK

BEGIN_EXTERN_FORTRAN
EXTERN_FSUBROUTINE(reservoirfortrancalc)(FINT*,FREAL8*,FREAL8*,FREAL8*,FREAL8*,FREAL8*,FREAL8*);
END_EXTERN_FORTRAN

// =====================================================================
// =====================================================================


/**

 */
class ReservoirFortranFunction : public openfluid::base::PluggableFunction
{
private:

	int m_nCouche;
	// IDVectorValueMap : vecteurs spatialises
	openfluid::core::IDDoubleValueMap m_th_cc;
	openfluid::core::IDVectorValueMap m_thCI;
	openfluid::core::IDVectorValueMap m_dz;

public:


	ReservoirFortranFunction(): PluggableFunction()
	{


	}


	// =====================================================================
	// =====================================================================


	~ReservoirFortranFunction()
	{


	}


	// =====================================================================
	// =====================================================================


	bool initParams(openfluid::core::FuncParamsMap_t /*Params*/)
	{


		return true;
	}

	// =====================================================================
	// =====================================================================


	bool prepareData()
	{


		return true;
	}


	// =====================================================================
	// =====================================================================


	bool checkConsistency()
	{


		return true;
	}


	// =====================================================================
	// =====================================================================


	bool initializeRun(const openfluid::base::SimulationInfo* /*SimInfo*/)
	{

		long ID;
		openfluid::core::Unit* SU;
		openfluid::core::StringValue str_thetai;
		openfluid::core::StringValue str_dz;
		openfluid::core::DoubleValue th_cc;
		DECLARE_UNITS_ORDERED_LOOP(1);
		bool lErr;

		BEGIN_UNITS_ORDERED_LOOP(1,"SU",SU)
		ID = SU->getID();

		OPENFLUID_GetInputData(SU,"th_cc",th_cc);
		OPENFLUID_GetInputData(SU,"thetai",str_thetai);
		OPENFLUID_GetInputData(SU,"dz",str_dz);

		openfluid::core::VectorValue thetaiVect;
		openfluid::core::VectorValue dzVect;
		str_thetai.toVectorValue(";",thetaiVect);
		str_dz.toVectorValue(";",dzVect);
		m_nCouche = thetaiVect.getSize();

		// pour fortran ThetaiVect.getData()
		// pour Fortran ThetaiVect.getSize()

		// creation des vecteurs déclarés comme attributs
		m_th_cc[ID] = th_cc;
		m_dz[ID]    = openfluid::core::VectorValue(m_nCouche);
		m_thCI[ID]  = openfluid::core::VectorValue(m_nCouche);

		for (int i=0;i<m_nCouche;i++)
			m_thCI[ID][i] = thetaiVect[i];

		for (int i=0;i<m_nCouche;i++)
			m_dz[ID][i] = dzVect[i];

		END_LOOP;

		return true;
	}

	// =====================================================================
	// =====================================================================


	bool runStep(const openfluid::base::SimulationStatus* SimStatus)
	{

		long ID;
		int CurrentStep;
		int PreviousStep;
		int k;
		double apportActu,drainage,storage;
		openfluid::core::VectorValue Theta;
		openfluid::core::Unit *SU;
		DECLARE_UNITS_ORDERED_LOOP(1) ;

		// definition des pas de temps en courant et precedent
		CurrentStep  = SimStatus->getCurrentStep();
		PreviousStep = CurrentStep-1;

		// debut de la boucle d espace
		BEGIN_UNITS_ORDERED_LOOP(1,"SU",SU)

		ID = SU->getID();

		OPENFLUID_GetVariable(SU,("water.atm-surf.H.rain"),CurrentStep,&apportActu);

		if (SimStatus->isFirstStep())
			drainage = 0;
		else
			OPENFLUID_GetVariable(SU,("water.surf-uz.drainage"),PreviousStep,&drainage);

		// calculs sur un pas de temps

		double *tmp_dz   = m_dz[ID].getData();
		double *tmp_thCI = m_thCI[ID].getData();
		double tmp_th_cc = m_th_cc[ID].get();

		CALL_FSUBROUTINE(reservoirfortrancalc)(&m_nCouche,&tmp_th_cc,&tmp_thCI[0],&tmp_dz[0],&apportActu,&storage,&drainage);

		Theta = openfluid::core::VectorValue(m_nCouche);
		for (k=0;k<m_nCouche;k++)
		{
			Theta[k] = tmp_thCI[k];
		}

		OPENFLUID_AppendVariable(SU,"water.surf-uz.theta",Theta);
		OPENFLUID_AppendVariable(SU,"water.surf-uz.storage",storage);
		OPENFLUID_AppendVariable(SU,"water.surf-uz.drainage",drainage);

		END_LOOP;
		return true;
	}

	// =====================================================================
	// =====================================================================


	bool finalizeRun(const openfluid::base::SimulationInfo* /*SimInfo*/)
	{


		return true;
	}

};


// =====================================================================
// =====================================================================


DEFINE_FUNCTION_HOOK(ReservoirFortranFunction);

